// NOTE: Assertions have been autogenerated by utils/update_cc_test_checks.py UTC_ARGS: --version 5
// RUN: %clang_cc1 -std=c2y -emit-llvm -o - %s | FileCheck %s

// This tests the non-VLA codegen behavior for _Countof.

typedef typeof(sizeof(0)) size_t;

// CHECK-LABEL: define{{( dso_local)?}} i{{64|32}} @test1(
// CHECK-SAME: ) #[[ATTR0:[0-9]+]] {
// CHECK-NEXT:  [[ENTRY:.*:]]
// CHECK-NEXT:    [[ARRAY:%.*]] = alloca [12 x i32], align
// CHECK-NEXT:    ret i{{64|32}} 12
//
size_t test1() {
  int array[12];
  return _Countof(array);
}

// CHECK-LABEL: define{{( dso_local)?}} i{{64|32}} @test2(
// CHECK-SAME: ) #[[ATTR0]] {
// CHECK-NEXT:  [[ENTRY:.*:]]
// CHECK-NEXT:    ret i{{64|32}} 100
//
size_t test2() {
  return _Countof(float[100]);
}
